#!/usr/bin/perl -w 

package esmith::FormMagick::Panel::webfilter;

use strict;

use esmith::FormMagick;
use esmith::ConfigDB;
use esmith::cgi;
use Exporter;
use Carp;

our @ISA = qw(esmith::FormMagick);

our $db = esmith::ConfigDB->open() || die "Couldn't open config db";

sub new {
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}

sub get_prop{
    my ($self,$entrie,$prop) = @_;
    return $db->get_prop($entrie, $prop) || '';
}

sub get_list{
    my ($self,$entrie,$prop) = @_;
    my $values = get_prop($self,$entrie,$prop);
    $values =~ s/[;,]/\n/g;
    return $values;
}

sub get_wbl{
    my ($self,$bl) = @_;
    open FILE, "</var/squidGuard/blacklists/$bl/domains" || die "Couldn't open file /var/squidGuard/blacklists/$bl/domains: $!\n";
    my $content = '';
    while (<FILE>){
        $content .= $_;
    }
    close FILE;
    open FILE, "</var/squidGuard/blacklists/$bl/urls" || die "Couldn't open file /var/squidGuard/blacklists/$bl/urls: $!\n";
    while (<FILE>){
        $content .= $_;
    }
    close FILE;
    return $content;
}

sub print_section_bar{
    my ($fm) = @_;
    print "  <tr>\n    <td colspan='2'>\n";
    print "<hr class=\"sectionbar\"/>\n";
    return undef;
}

sub print_categories{
    my ($fm) = @_;
    my $q = $fm->{cgi};
    my @blockedCategories = split(/[,;]/, (get_prop($fm, 'squidguard', 'BlockedCategories')));
    print "<tr><td class=\"sme-noborders-label\">",
        $fm->localise('BLOCKED_CATEGORIES'),
        "</td><td>\n";
    print $q->start_table({-class => "sme-border"}),"\n";
    print $q->Tr(
        esmith::cgi::genSmallCell($q, $fm->localise('BLOCKED_OR_NOT'),"header"),
        esmith::cgi::genSmallCell($q, $fm->localise('CATEGORY'),"header"),
        esmith::cgi::genSmallCell($q, $fm->localise('DESCRIPTION'),"header")
    );
    my @categories = ();
    my @hide = split(/[;,]/, (get_prop($fm, 'squidguard', 'DisabledCategories')));
    opendir BL, '/var/squidGuard/blacklists' || die "Couldn't open blacklists directory\n";
    while (my $cat = readdir(BL)){
        next if (!-d "/var/squidGuard/blacklists/$cat" ||
                $cat =~ /^\./ ||
                $cat eq 'black' ||
                $cat eq 'white' ||
                grep { $_ eq $cat } @hide);
        push @categories, $cat;
    }
    closedir BL;
    foreach my $c (sort @categories){
        my $checked = '';
        if (grep { $_ eq $c } @blockedCategories) {
            $checked = 'checked';
        }
        print $q->Tr(
            $q->td(
                "<input type=\"checkbox\""
                . " name=\"blockedCategories\""
                . " $checked value=\"$c\">"
            ),
            esmith::cgi::genSmallCell($q, $c,"normal"),
            esmith::cgi::genSmallCell( $q, $fm->localise($c),"normal")
        );
    }
    print "</table></td></tr>\n";
    return undef;
}

sub change_settings {
    my $fm = shift;
    my $q  = $fm->{'cgi'};

    $db->set_prop('squidguard', 'status', $q->param('url_filter_status'));
    $db->set_prop('squidclamav', 'status', $q->param('av_filter_status'));

    my $squid = $db->get('squid') || die "Couldn't find squid service in configuration DB\n";
    my $squidstatus = $squid->prop('status') || 'disabled';

    # If either squidguard or havp is enabled, squid needs to be enabled too
    if ((($q->param('url_filter_status') eq 'enabled') || ($q->param('av_filter_status') eq 'enabled')) &&
            ($squidstatus ne 'enabled')){
        $db->set_prop('squid', 'status', 'enabled');
        $fm->error('ERROR') unless (system ("/sbin/e-smith/signal-event proxy-update") == 0 );
    }

    $db->set_prop('squidguard', 'BlockedCategories', join(',', $q->param('blockedCategories')));
    my $ip = $q->param('unrestricted');
    $ip =~ s/\r?\n/,/g;
    $db->set_prop('squidguard', 'UnrestrictedIP', $ip);

    # Split domains and URL
    # as squidguard wants them in their own file
    my %list; 
    $list{black_domains} = [ grep {$_ !~ m|/|} split(/\r?\n/, $q->param('black')) ];
    $list{black_urls}     = [ grep {$_ =~ m|/|} split(/\r?\n/, $q->param('black')) ];
    $list{white_domains} = [ grep {$_ !~ m|/|} split(/\r?\n/, $q->param('white')) ];
    $list{white_urls}    = [ grep {$_ =~ m|/|} split(/\r?\n/, $q->param('white')) ];

    foreach (qw{black_domains black_urls white_domains white_urls}){
        my $file = $_;
        my $bl = $file;
        $file =~ s!_!/!;
        open FILE, ">/var/squidGuard/blacklists/$file" || die "Couldn't open $file: $!\n";
        foreach (@{$list{$bl}}){
            $_ =~ s|https?://||g;
            print FILE $_."\n";
        }
        close FILE;
    }
    # Write whitelist for squidclamav
    open FILE, ">/etc/squid/clamav_whitelist" || die "Couldn't open /etc/squid/clamav_whitelist: $!\n";
    foreach (@{$list{white_domains}},@{$list{white_urls}}){
        $_ =~ s|https?://||g;
        $_ =~ s|^(.*)|whitelist\ \.$1|g;
        $_ =~ s|\.|\\\.|g;
        print FILE $_."\n";
    }
    close FILE;

    unless (system ("/sbin/e-smith/signal-event http-proxy-update") == 0 ){
        $fm->error('ERROR');
    }

    $fm->success('SUCCESS');
}

sub valid_ip_list{
    my ($fm,$text) = @_;
    foreach (split /\r?\n/, $text){
      return $fm->localise('NOT_A_VALID_IP_LIST') unless
          (valid_ip_or_range($fm,$_) eq 'OK' or $_ eq '');
    }
    return 'OK'
}

sub valid_ip_or_range{
    my ($fm,$entry) = @_;
    if ($entry =~ m/^\d+\.\d+\.\d+\.\d+$/){
        return CGI::FormMagick::Validator::ip_number($fm, $entry)
    }
    elsif ($entry =~ m/^(\d+\.\d+\.\d+\.\d+)[\/\-](\d+\.\d+\.\d+\.\d+)$/){
        return 'OK' if (CGI::FormMagick::Validator::ip_number($fm, $1) eq 'OK' &&
                        CGI::FormMagick::Validator::ip_number($fm, $2) eq 'OK');
    }
    elsif ($entry =~ m/^(\d+\.\d+\.\d+\.\d+)\/(\d{1,2})$/){
        return 'OK' if (CGI::FormMagick::Validator::ip_number($fm, $1) eq 'OK' &&
                        $2 < 32 &&
                        $2 > 1);
    }
    return $fm->localise('NOT_A_VALID_IP_OR_RANGE');
}


1;

